import { world, system, ItemStack } from '@minecraft/server'

world.beforeEvents.worldInitialize.subscribe(initEvent => {
    initEvent.blockComponentRegistry.registerCustomComponent('v360:block_destroy', {
        onPlayerDestroy: (data) => {
            const player = data.player
            const block = data.block
            const spawnItemStack = new ItemStack(data.destroyedBlockPermutation.type.id, 1);
            const selectedItem = player.getComponent('equippable').getEquipment('Mainhand');
            const isPickaxe = selectedItem && selectedItem.hasTag('minecraft:is_pickaxe');
            if (isPickaxe && player.getGameMode() !== "creative") {
                system.run(() => {
                    block.dimension.spawnItem(spawnItemStack, block.center())
                })
            }
        }
    })
})

world.beforeEvents.worldInitialize.subscribe(initEvent => {
    initEvent.blockComponentRegistry.registerCustomComponent('v360:double_block_destroy', {
        onPlayerDestroy: (data) => {
            const player = data.player
            const block = data.block
            const spawnItemStack = new ItemStack(data.destroyedBlockPermutation.type.id, 1);
            const selectedItem = player.getComponent('equippable').getEquipment('Mainhand');
            const isPickaxe = selectedItem && selectedItem.hasTag('minecraft:is_pickaxe');
            if (isPickaxe && player.getGameMode() !== "creative") {
                system.run(() => {
                    block.dimension.spawnItem(spawnItemStack, block.center())
                })
            }
        }
    })
})

world.beforeEvents.worldInitialize.subscribe(initEvent => {
    initEvent.blockComponentRegistry.registerCustomComponent('v360:stone_double_block_destroy', {
        onPlayerDestroy: (data) => {
            const player = data.player
            const block = data.block
            const spawnItemStack = new ItemStack(data.destroyedBlockPermutation.type.id, 1);
            const selectedItem = player.getComponent('equippable').getEquipment('Mainhand');
            const isPickaxe = selectedItem && selectedItem.hasTag('minecraft:is_pickaxe');
            if (!selectedItem?.typeId.includes('wooden') && isPickaxe && player.getGameMode() !== "creative") {
                system.run(() => {
                    block.dimension.spawnItem(spawnItemStack, block.center())
                })
            }
        }
    })
})

world.beforeEvents.worldInitialize.subscribe(initEvent => {
    initEvent.blockComponentRegistry.registerCustomComponent('v360:non_stone_double_block_destroy', {
        onPlayerDestroy: (data) => {
            const player = data.player
            const block = data.block
            const spawnItemStack = new ItemStack(data.destroyedBlockPermutation.type.id, 2);
            if (player.getGameMode() !== "creative") {
                system.run(() => {
                    block.dimension.spawnItem(spawnItemStack, block.center())
                })
            }
        }
    })
})

world.afterEvents.playerBreakBlock.subscribe(data => {
    const brokenBlock = data.brokenBlockPermutation
    const player = data.player
    const block = data.block
    const selectedItem = player.getComponent('equippable').getEquipment('Mainhand')
    const blockItem = data.dimension.getEntities({ type: 'minecraft:item', location: block.center(), maxDistance: 0.6 })
    if (player.getGameMode() !== 'creative') {
        if (brokenBlock.hasTag('minecraft:wood_tier_destructible') && brokenBlock.hasTag('minecraft:is_pickaxe_item_destructible') && (!selectedItem?.hasTag('minecraft:is_pickaxe'))) {
            blockItem[0].kill()
        }
        if (brokenBlock.hasTag('minecraft:stone_tier_destructible') && brokenBlock.hasTag('minecraft:is_pickaxe_item_destructible') && (!selectedItem?.hasTag('minecraft:is_pickaxe') || (selectedItem?.typeId.includes('minecraft:wooden')))) {
            blockItem[0].kill()
        }
    }
})

world.afterEvents.playerBreakBlock.subscribe(data => {
    const block = data.block
    const { x, y, z } = block.location
    if (block.above().permutation.matches('v360:citrine_cluster', { 'minecraft:block_face': 'up' }) || block.above().permutation.matches('v360:large_citrine_bud', { 'minecraft:block_face': 'up' }) || block.above().permutation.matches('v360:medium_citrine_bud', { 'minecraft:block_face': 'up' }) || block.above().permutation.matches('v360:small_citrine_bud', { 'minecraft:block_face': 'up' }) || block.above().permutation.matches('v360:jade_cluster', { 'minecraft:block_face': 'up' }) || block.above().permutation.matches('v360:large_jade_bud', { 'minecraft:block_face': 'up' }) || block.above().permutation.matches('v360:medium_jade_bud', { 'minecraft:block_face': 'up' }) || block.above().permutation.matches('v360:small_jade_bud', { 'minecraft:block_face': 'up' }) || block.above().permutation.matches('v360:garnet', { 'minecraft:block_face': 'up' }) || block.above().permutation.matches('v360:large_garnet_bud', { 'minecraft:block_face': 'up' }) || block.above().permutation.matches('v360:medium_garnet_bud', { 'minecraft:block_face': 'up' }) || block.above().permutation.matches('v360:small_garnet_bud', { 'minecraft:block_face': 'up' }) || block.above().typeId === 'v360:garpite' || block.above().typeId === 'v360:glow_stalagmit' || block.above().typeId === 'v360:rainbow_cluster' || block.north().permutation.matches('v360:copper_torch', { 'minecraft:block_face': 'up' })) {
        block.dimension.runCommand(`fill ${x} ${y + 1} ${z} ${x} ${y + 1} ${z} air [] destroy`)
    }
    if (block.below().permutation.matches('v360:citrine_cluster', { 'minecraft:block_face': 'down' }) || block.below().permutation.matches('v360:large_citrine_bud', { 'minecraft:block_face': 'down' }) || block.below().permutation.matches('v360:medium_citrine_bud', { 'minecraft:block_face': 'down' }) || block.below().permutation.matches('v360:small_citrine_bud', { 'minecraft:block_face': 'down' }) || block.below().permutation.matches('v360:jade_cluster', { 'minecraft:block_face': 'down' }) || block.below().permutation.matches('v360:large_jade_bud', { 'minecraft:block_face': 'down' }) || block.below().permutation.matches('v360:medium_jade_bud', { 'minecraft:block_face': 'down' }) || block.below().permutation.matches('v360:small_jade_bud', { 'minecraft:block_face': 'down' }) || block.below().permutation.matches('v360:garnet', { 'minecraft:block_face': 'down' }) || block.below().permutation.matches('v360:large_garnet_bud', { 'minecraft:block_face': 'down' }) || block.below().permutation.matches('v360:medium_garnet_bud', { 'minecraft:block_face': 'down' }) || block.below().permutation.matches('v360:small_garnet_bud', { 'minecraft:block_face': 'down' }) || block.below().typeId === 'v360:gargot' || block.below().typeId === 'v360:tall_gargot' || block.below().typeId === 'v360:glow_stalagtit') {
        block.dimension.runCommand(`fill ${x} ${y - 1} ${z} ${x} ${y - 1} ${z} air [] destroy`)
    }
    if (block.north().permutation.matches('v360:citrine_cluster', { 'minecraft:block_face': 'north' }) || block.north().permutation.matches('v360:large_citrine_bud', { 'minecraft:block_face': 'north' }) || block.north().permutation.matches('v360:medium_citrine_bud', { 'minecraft:block_face': 'north' }) || block.north().permutation.matches('v360:small_citrine_bud', { 'minecraft:block_face': 'north' }) || block.north().permutation.matches('v360:jade_cluster', { 'minecraft:block_face': 'north' }) || block.north().permutation.matches('v360:large_jade_bud', { 'minecraft:block_face': 'north' }) || block.north().permutation.matches('v360:medium_jade_bud', { 'minecraft:block_face': 'north' }) || block.north().permutation.matches('v360:small_jade_bud', { 'minecraft:block_face': 'north' }) || block.north().permutation.matches('v360:garnet', { 'minecraft:block_face': 'north' }) || block.north().permutation.matches('v360:large_garnet_bud', { 'minecraft:block_face': 'north' }) || block.north().permutation.matches('v360:medium_garnet_bud', { 'minecraft:block_face': 'north' }) || block.north().permutation.matches('v360:small_garnet_bud', { 'minecraft:block_face': 'north' }) || block.north().permutation.matches('v360:blue_bonnets', { 'minecraft:cardinal_direction': 'south' }) || block.north().permutation.matches('v360:blue_bonnets_seeds', { 'minecraft:cardinal_direction': 'south' }) || block.north().permutation.matches('v360:blue_bonnets_waxed0', { 'minecraft:cardinal_direction': 'south' }) || block.north().permutation.matches('v360:blue_bonnets_waxed1', { 'minecraft:cardinal_direction': 'south' }) || block.north().permutation.matches('v360:copper_torch', { 'minecraft:block_face': 'south' }) || block.north().permutation.matches('v360:apples_and_dried_flowers_painting', { 'minecraft:cardinal_direction': 'south' }) || block.north().permutation.matches('v360:carving_wall_painting', { 'minecraft:cardinal_direction': 'south' }) || block.north().permutation.matches('v360:cave_of_skulls_painting', { 'minecraft:cardinal_direction': 'south' }) || block.north().permutation.matches('v360:the_dig_painting', { 'minecraft:cardinal_direction': 'south' })) {
        block.dimension.runCommand(`fill ${x} ${y} ${z - 1} ${x} ${y} ${z - 1} air [] destroy`)
    }
    if (block.south().permutation.matches('v360:citrine_cluster', { 'minecraft:block_face': 'south' }) || block.south().permutation.matches('v360:large_citrine_bud', { 'minecraft:block_face': 'south' }) || block.south().permutation.matches('v360:medium_citrine_bud', { 'minecraft:block_face': 'south' }) || block.south().permutation.matches('v360:small_citrine_bud', { 'minecraft:block_face': 'south' }) || block.south().permutation.matches('v360:jade_cluster', { 'minecraft:block_face': 'south' }) || block.south().permutation.matches('v360:large_jade_bud', { 'minecraft:block_face': 'south' }) || block.south().permutation.matches('v360:medium_jade_bud', { 'minecraft:block_face': 'south' }) || block.south().permutation.matches('v360:small_jade_bud', { 'minecraft:block_face': 'south' }) || block.south().permutation.matches('v360:garnet', { 'minecraft:block_face': 'south' }) || block.south().permutation.matches('v360:large_garnet_bud', { 'minecraft:block_face': 'south' }) || block.south().permutation.matches('v360:medium_garnet_bud', { 'minecraft:block_face': 'south' }) || block.south().permutation.matches('v360:small_garnet_bud', { 'minecraft:block_face': 'south' }) || block.south().permutation.matches('v360:blue_bonnets', { 'minecraft:cardinal_direction': 'north' }) || block.south().permutation.matches('v360:blue_bonnets_seeds', { 'minecraft:cardinal_direction': 'north' }) || block.south().permutation.matches('v360:blue_bonnets_waxed0', { 'minecraft:cardinal_direction': 'north' }) || block.south().permutation.matches('v360:blue_bonnets_waxed1', { 'minecraft:cardinal_direction': 'north' }) || block.south().permutation.matches('v360:copper_torch', { 'minecraft:block_face': 'south' }) || block.south().permutation.matches('v360:apples_and_dried_flowers_painting', { 'minecraft:cardinal_direction': 'north' }) || block.south().permutation.matches('v360:carving_wall_painting', { 'minecraft:cardinal_direction': 'north' }) || block.south().permutation.matches('v360:cave_of_skulls_painting', { 'minecraft:cardinal_direction': 'north' }) || block.south().permutation.matches('v360:the_dig_painting', { 'minecraft:cardinal_direction': 'north' })) {
        block.dimension.runCommand(`fill ${x} ${y} ${z + 1} ${x} ${y} ${z + 1} air [] destroy`)
    }
    if (block.west().permutation.matches('v360:citrine_cluster', { 'minecraft:block_face': 'west' }) || block.west().permutation.matches('v360:large_citrine_bud', { 'minecraft:block_face': 'west' }) || block.west().permutation.matches('v360:medium_citrine_bud', { 'minecraft:block_face': 'west' }) || block.west().permutation.matches('v360:small_citrine_bud', { 'minecraft:block_face': 'west' }) || block.west().permutation.matches('v360:jade_cluster', { 'minecraft:block_face': 'west' }) || block.west().permutation.matches('v360:large_jade_bud', { 'minecraft:block_face': 'west' }) || block.west().permutation.matches('v360:medium_jade_bud', { 'minecraft:block_face': 'west' }) || block.west().permutation.matches('v360:small_jade_bud', { 'minecraft:block_face': 'west' }) || block.west().permutation.matches('v360:garnet', { 'minecraft:block_face': 'west' }) || block.west().permutation.matches('v360:large_garnet_bud', { 'minecraft:block_face': 'west' }) || block.west().permutation.matches('v360:medium_garnet_bud', { 'minecraft:block_face': 'west' }) || block.west().permutation.matches('v360:small_garnet_bud', { 'minecraft:block_face': 'west' }) || block.west().permutation.matches('v360:blue_bonnets', { 'minecraft:cardinal_direction': 'east' }) || block.west().permutation.matches('v360:blue_bonnets_seeds', { 'minecraft:cardinal_direction': 'east' }) || block.west().permutation.matches('v360:blue_bonnets_waxed0', { 'minecraft:cardinal_direction': 'east' }) || block.west().permutation.matches('v360:blue_bonnets_waxed1', { 'minecraft:cardinal_direction': 'east' }) || block.west().permutation.matches('v360:copper_torch', { 'minecraft:block_face': 'west' }) || block.west().permutation.matches('v360:apples_and_dried_flowers_painting', { 'minecraft:cardinal_direction': 'east' }) || block.west().permutation.matches('v360:carving_wall_painting', { 'minecraft:cardinal_direction': 'east' }) || block.west().permutation.matches('v360:cave_of_skulls_painting', { 'minecraft:cardinal_direction': 'east' }) || block.west().permutation.matches('v360:the_dig_painting', { 'minecraft:cardinal_direction': 'east' })) {
        block.dimension.runCommand(`fill ${x - 1} ${y} ${z} ${x - 1} ${y} ${z} air [] destroy`)
    }
    if (block.east().permutation.matches('v360:citrine_cluster', { 'minecraft:block_face': 'east' }) || block.east().permutation.matches('v360:large_citrine_bud', { 'minecraft:block_face': 'east' }) || block.east().permutation.matches('v360:medium_citrine_bud', { 'minecraft:block_face': 'east' }) || block.east().permutation.matches('v360:small_citrine_bud', { 'minecraft:block_face': 'east' }) || block.east().permutation.matches('v360:jade_cluster', { 'minecraft:block_face': 'east' }) || block.east().permutation.matches('v360:large_jade_bud', { 'minecraft:block_face': 'east' }) || block.east().permutation.matches('v360:medium_jade_bud', { 'minecraft:block_face': 'east' }) || block.east().permutation.matches('v360:small_jade_bud', { 'minecraft:block_face': 'east' }) || block.east().permutation.matches('v360:garnet', { 'minecraft:block_face': 'east' }) || block.east().permutation.matches('v360:large_garnet_bud', { 'minecraft:block_face': 'east' }) || block.east().permutation.matches('v360:medium_garnet_bud', { 'minecraft:block_face': 'east' }) || block.east().permutation.matches('v360:small_garnet_bud', { 'minecraft:block_face': 'east' }) || block.east().permutation.matches('v360:blue_bonnets', { 'minecraft:cardinal_direction': 'west' }) || block.east().permutation.matches('v360:blue_bonnets_seeds', { 'minecraft:cardinal_direction': 'west' }) || block.east().permutation.matches('v360:blue_bonnets_waxed0', { 'minecraft:cardinal_direction': 'west' }) || block.east().permutation.matches('v360:blue_bonnets_waxed1', { 'minecraft:cardinal_direction': 'west' }) || block.east().permutation.matches('v360:copper_torch', { 'minecraft:block_face': 'east' }) || block.east().permutation.matches('v360:apples_and_dried_flowers_painting', { 'minecraft:cardinal_direction': 'west' }) || block.east().permutation.matches('v360:carving_wall_painting', { 'minecraft:cardinal_direction': 'west' }) || block.east().permutation.matches('v360:cave_of_skulls_painting', { 'minecraft:cardinal_direction': 'west' }) || block.east().permutation.matches('v360:the_dig_painting', { 'minecraft:cardinal_direction': 'west' })) {
        block.dimension.runCommand(`fill ${x + 1} ${y} ${z} ${x + 1} ${y} ${z} air [] destroy`)
    }
})
