import { world, BlockPermutation, ItemStack } from '@minecraft/server'

const blockStates = new Map()

world.beforeEvents.worldInitialize.subscribe(initEvent => {
    initEvent.blockComponentRegistry.registerCustomComponent('v360:bronze_lamp', {
        onTick(e) {
            const { block } = e
            const { x, y, z } = block.location
            const blockKey = `${x},${y},${z}`
            const currentState = block.permutation.getState('v360:lit')
            const adjacentBlocks = {
                north: block.north(),
                east: block.east(),
                south: block.south(),
                west: block.west(),
                above: block.above(),
                below: block.below()
            }
            const excludedBlocks = [
                block.typeId.includes('door'), block.typeId.includes('trapdoor'), 'minecraft:observer', 'minecraft:unpowered_repeater', 'minecraft:lit_redstone_lamp',
                'minecraft:powered_repeater', 'minecraft:unpowered_comparator', 'minecraft:powered_comparator'
            ]
            const hasRedstonePower = Object.values(adjacentBlocks).some(adjacentBlock =>
                !excludedBlocks.includes(adjacentBlock?.typeId) && (adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 1 }) || adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 2 }) || adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 3 }) || adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 4 }) || adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 5 }) || adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 6 }) || adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 7 }) || adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 8 }) || adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 9 }) || adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 10 }) || adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 11 }) || adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 12 }) || adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 13 }) || adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 14 }) || adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 15 }) || adjacentBlock?.typeId === 'minecraft:redstone_block' || adjacentBlock?.typeId === 'minecraft:redstone_torch') || adjacentBlock?.permutation.matches('lever', { 'open_bit': true }) || adjacentBlock?.permutation.matches('tripwire_hook', { 'powered_bit': true }) || adjacentBlock?.permutation.matches('detector_rail', { 'rail_data_bit': true })
            )
            const isRedstoneTorchTop = adjacentBlocks.above?.typeId === 'minecraft:redstone_torch' &&
                adjacentBlocks.above.permutation.getState('torch_facing_direction') === 'top'
            const destroyableBlocksArray = [
                block.typeId.includes('button'), block.typeId.includes('pressure_plate')
            ]
            const destroyableBlocks = destroyableBlocksArray.includes(adjacentBlocks.above?.typeId)
            const observerFacingBlock = [
                { block: adjacentBlocks.north, direction: 'north' },
                { block: adjacentBlocks.east, direction: 'east' },
                { block: adjacentBlocks.south, direction: 'south' },
                { block: adjacentBlocks.west, direction: 'west' },
                { block: adjacentBlocks.above, direction: 'up' },
                { block: adjacentBlocks.below, direction: 'down' }
            ].some(({ block, direction }) => block?.typeId === 'minecraft:observer' &&
                block?.permutation.getState('minecraft:facing_direction') === direction && block?.permutation.matches('observer', { 'powered_bit': true }))
            const poweredRepeater = [
                { block: adjacentBlocks.north, direction: 'north' },
                { block: adjacentBlocks.east, direction: 'east' },
                { block: adjacentBlocks.south, direction: 'south' },
                { block: adjacentBlocks.west, direction: 'west' }
            ].some(({ block, direction }) => block?.typeId === 'minecraft:powered_repeater' &&
                block?.permutation.getState('minecraft:cardinal_direction') === direction)
            const poweredComparator = [
                { block: adjacentBlocks.north, direction: 'north' },
                { block: adjacentBlocks.east, direction: 'east' },
                { block: adjacentBlocks.south, direction: 'south' },
                { block: adjacentBlocks.west, direction: 'west' }
            ].some(({ block, direction }) => block?.typeId === 'minecraft:powered_comparator' &&
                block?.permutation.getState('minecraft:cardinal_direction') === direction)
            const previousState = blockStates.get(blockKey) || false
            const shouldOpen = (hasRedstonePower || observerFacingBlock || poweredRepeater || poweredComparator) && !previousState && !isRedstoneTorchTop;
            const shouldClose = (!hasRedstonePower && !observerFacingBlock && !poweredRepeater && !poweredComparator) && previousState && !isRedstoneTorchTop
            if (shouldOpen || shouldClose) {
                const newState = shouldOpen;
                block.setPermutation(block.permutation.withState('v360:lit', newState))
                blockStates.set(blockKey, newState)
                if (destroyableBlocks) {
                    block.dimension.runCommand(`setblock ${x} ${y + 1} ${z} air destroy`)
                }
            }
        }
    })
})

world.beforeEvents.worldInitialize.subscribe(initEvent => {
    initEvent.blockComponentRegistry.registerCustomComponent('v360:bronze_grate', {
        onPlayerInteract(e) {
            const { block, player } = e
            const equipment = player.getComponent('equippable')
            const selectedItem = equipment.getEquipment('Mainhand')
            if (selectedItem?.typeId === 'minecraft:water_bucket') {
                player.playSound('bucket.empty_water')
                if (player.getGameMode() !== "creative") {
                    equipment.setEquipment('Mainhand', new ItemStack('minecraft:bucket', 1));
                }
            }
            if (block.typeId === 'v360:bronze_grate' && selectedItem?.typeId === 'minecraft:water_bucket') {
                const structureName = 'mystructure:bronze_grate'
                const { x, y, z } = block
                world.structureManager.place(structureName, e.dimension, { x, y, z })
            }
        }
    })
})

world.beforeEvents.worldInitialize.subscribe(eventData => {
    eventData.blockComponentRegistry.registerCustomComponent('v360:bronze_trapdoor', {
        onPlayerInteract(e) {
            const { block, player } = e;
            const equipment = player.getComponent('equippable')
            const selectedItem = equipment.getEquipment('Mainhand')
            const currentState = block.permutation.getState('v360:open')
            const newOpenState = !currentState
            const newPermutation = BlockPermutation.resolve(block.typeId, {
                ...block.permutation.getAllStates(),
                'v360:open': newOpenState
            })
            block.setPermutation(newPermutation)
            const sound = currentState ? 'open_trapdoor.copper' : 'open_trapdoor.copper'
            player.playSound(sound)
            if (selectedItem?.typeId === 'minecraft:water_bucket') {
                player.playSound('bucket.empty_water')
                if (player.getGameMode() !== "creative") {
                    equipment.setEquipment('Mainhand', new ItemStack('minecraft:bucket', 1));
                }
            }
            if (block.typeId === 'v360:bronze_trapdoor' && selectedItem?.typeId === 'minecraft:water_bucket') {
                const currentStates = block.permutation.getAllStates()
                const structureName = 'mystructure:bronze_trapdoor'
                const { x, y, z } = block.location
                world.structureManager.place(structureName, e.dimension, { x, y, z })
                const newBlock = e.dimension.getBlock({ x, y, z })
                const newStates = { ...newBlock.permutation.getAllStates(), ...currentStates }
                const newPermutation = BlockPermutation.resolve(newBlock.typeId, newStates)
                newBlock.setPermutation(newPermutation);
            }
        },
        onTick(e) {
            const { block } = e
            const { x, y, z } = block.location
            const blockKey = `${x},${y},${z}`
            const currentState = block.permutation.getState('v360:open')
            const sound = currentState ? 'open_trapdoor.copper' : 'open_trapdoor.copper'
            const adjacentBlocks = {
                north: block.north(),
                east: block.east(),
                south: block.south(),
                west: block.west(),
                above: block.above(),
                below: block.below()
            }
            const excludedBlocks = [
                block.typeId.includes('door'), block.typeId.includes('trapdoor'), 'minecraft:observer', 'minecraft:unpowered_repeater',
                'minecraft:powered_repeater', 'minecraft:unpowered_comparator', 'minecraft:powered_comparator'
            ]

            // Check if any adjacent blocks have redstone power, excluding certain block types
            const hasRedstonePower = Object.values(adjacentBlocks).some(adjacentBlock =>
                !excludedBlocks.includes(adjacentBlock?.typeId) && (adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 1 }) || adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 2 }) || adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 3 }) || adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 4 }) || adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 5 }) || adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 6 }) || adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 7 }) || adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 8 }) || adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 9 }) || adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 10 }) || adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 11 }) || adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 12 }) || adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 13 }) || adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 14 }) || adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 15 }) || adjacentBlock?.typeId === 'minecraft:redstone_block' || adjacentBlock?.typeId === 'minecraft:redstone_torch') || adjacentBlock?.permutation.matches('lever', { 'open_bit': true }) || adjacentBlock?.permutation.matches('tripwire_hook', { 'powered_bit': true }) || adjacentBlock?.permutation.matches('detector_rail', { 'rail_data_bit': true })
            )

            // Check if there's a redstone torch directly above the block
            const isRedstoneTorchTop = adjacentBlocks.above?.typeId === 'minecraft:redstone_torch' &&
                adjacentBlocks.above.permutation.getState('torch_facing_direction') === 'top';

            // Define blocks that should be destroyed when receiving redstone power and in a specific position
            const destroyableBlocksArray = [
                block.typeId.includes('button'), block.typeId.includes('pressure_plate')
            ];

            // Check if the block above is a destroyable block and is powered
            const destroyableBlocks = destroyableBlocksArray.includes(adjacentBlocks.above?.typeId)

            // Check if an adjacent observer is facing the block and is powered
            const observerFacingBlock = [
                { block: adjacentBlocks.north, direction: 'north' },
                { block: adjacentBlocks.east, direction: 'east' },
                { block: adjacentBlocks.south, direction: 'south' },
                { block: adjacentBlocks.west, direction: 'west' },
                { block: adjacentBlocks.above, direction: 'up' },
                { block: adjacentBlocks.below, direction: 'down' }
            ].some(({ block, direction }) => block?.typeId === 'minecraft:observer' &&
                block?.permutation.getState('minecraft:facing_direction') === direction && block?.permutation.matches('observer', { 'powered_bit': true }));

            // Check if an adjacent powered repeater is facing the block
            const poweredRepeater = [
                { block: adjacentBlocks.north, direction: 'north' },
                { block: adjacentBlocks.east, direction: 'east' },
                { block: adjacentBlocks.south, direction: 'south' },
                { block: adjacentBlocks.west, direction: 'west' }
            ].some(({ block, direction }) => block?.typeId === 'minecraft:powered_repeater' &&
                block?.permutation.getState('minecraft:cardinal_direction') === direction);

            // Check if an adjacent powered comparator is facing the block
            const poweredComparator = [
                { block: adjacentBlocks.north, direction: 'north' },
                { block: adjacentBlocks.east, direction: 'east' },
                { block: adjacentBlocks.south, direction: 'south' },
                { block: adjacentBlocks.west, direction: 'west' }
            ].some(({ block, direction }) => block?.typeId === 'minecraft:powered_comparator' &&
                block?.permutation.getState('minecraft:cardinal_direction') === direction);

            // Retrieve the previous state of the block from the Map
            const previousState = blockStates.get(blockKey) || false;

            const shouldOpen = (hasRedstonePower || observerFacingBlock || poweredRepeater || poweredComparator) && !previousState && !isRedstoneTorchTop;
            const shouldClose = (!hasRedstonePower && !observerFacingBlock && !poweredRepeater && !poweredComparator) && previousState && !isRedstoneTorchTop
            if (shouldOpen || shouldClose) {
                const newState = shouldOpen;
                block.setPermutation(block.permutation.withState('v360:open', newState))
                block.dimension.playSound(sound, block.location)
                blockStates.set(blockKey, newState)
                if (destroyableBlocks) {
                    block.dimension.runCommand(`setblock ${x} ${y + 1} ${z} air destroy`)
                }
            }
        }
    })
})