import { world, system } from '@minecraft/server'

world.beforeEvents.worldInitialize.subscribe(initEvent => {
    initEvent.blockComponentRegistry.registerCustomComponent('v360:copper_fire', {
        onTick({ block }) {
            const chance = Math.random()
            const { x, y, z } = block.location
            if (chance < 0.05) {
                block.dimension.spawnParticle('v360:copper_fire_smoke', block.bottomCenter())
            }
            if (chance < 0.01) {
                block.dimension.playSound('fire.fire', block.location)
            }
        }
    })
})

world.afterEvents.itemUseOn.subscribe(data => {
    const block = data.block
    const player = data.source
    const face = data.blockFace
    const item = data.itemStack
    const equipment = player.getComponent('equippable')
    if (block?.typeId.includes('copper_barrel') && face === 'Up' && (item?.typeId === 'minecraft:flint_and_steel' || item?.typeId === 'minecraft:fire_charge')) {
        if (block?.above().isAir) {
            system.run(() => {
                block.above().setType('v360:copper_fire')
                block.dimension.playSound('fire.ignite', block.location)
                if (item?.typeId === 'minecraft:flint_and_steel') {
                    const inventory = player.getComponent('inventory').container
                    const damage = item.getComponent("durability").damage
                    const durability = item.getComponent("durability").maxDurability
                    if (damage >= durability) {
                        inventory.setItem(player.item, null)
                        player.playSound("random.break");
                        return;
                    }
                    else {
                        item.getComponent("durability").damage += 1
                        equipment.setEquipment('Mainhand', item)
                    }
                }
            })
        }
    }
})

world.afterEvents.projectileHitBlock.subscribe(data => {
    const block = data.getBlockHit().block
    const projectile = data.projectile
    if ((projectile?.typeId === 'minecraft:splash_potion' || projectile?.typeId === 'minecraft:lingering_potion') && block?.above().typeId === 'v360:copper_fire') {
        block.above().setType('air')
    }
})

system.runInterval(() => {
    world.getDimension('overworld' || 'nether' || 'the_end').getEntities().filter(
        (data) => {
            try {
                const { x, y, z } = data.location
                if (data?.dimension?.getBlock({ x: x, y: y, z: z })?.typeId === 'v360:copper_fire' && (!data?.hasComponent('fire_immune')) && !data?.getEffect('fire_resistance')?.isValid) {
                    data?.applyDamage(1)
                    data?.setOnFire(8)
                }
            } catch { }
        }
    )
}, 2)