import { world, system, ItemStack } from '@minecraft/server'

world.beforeEvents.worldInitialize.subscribe(initEvent => {
    initEvent.blockComponentRegistry.registerCustomComponent('v360:crystal_lamp', {
        onTick({ block }) {
            const chance = Math.random()
            const state0 = block.permutation.withState('v360:light', 0)
            const state1 = block.permutation.withState('v360:light', 1)
            const state2 = block.permutation.withState('v360:light', 2)
            const state3 = block.permutation.withState('v360:light', 3)
            const state4 = block.permutation.withState('v360:light', 4)
            if (block.hasTag('v360:crystal_lamp4')) {
                block.setPermutation(state1)
                block.dimension.playSound('block.copper_bulb.turn_off', block.location, { volume: 3, pitch: 1.4 })
            }
            else if (block.hasTag('v360:crystal_lamp3')) {
                block.setPermutation(state2)
                block.dimension.playSound('block.copper_bulb.turn_off', block.location, { volume: 3, pitch: 1.3 })
            }
            else if (block.hasTag('v360:crystal_lamp2')) {
                block.setPermutation(state3)
                block.dimension.playSound('block.copper_bulb.turn_off', block.location, { volume: 3, pitch: 1.2 })
            }
            else if (block.hasTag('v360:crystal_lamp1')) {
                block.setPermutation(state4)
                block.dimension.playSound('block.copper_bulb.turn_off', block.location, { volume: 3, pitch: 1.1 })
                if (chance < 0.15) {
                    if (block?.typeId === 'v360:amethyst_lamp') {
                        block.dimension.spawnItem(new ItemStack('minecraft:amethyst_shard', 1), block.center())
                    }
                    else if (block?.typeId === 'v360:citrine_lamp') {
                        block.dimension.spawnItem(new ItemStack('v360:citrine_shard', 1), block.center())
                    }
                    else if (block?.typeId === 'v360:garnet_lamp') {
                        block.dimension.spawnItem(new ItemStack('v360:garnet_shard', 1), block.center())
                    }
                    else if (block?.typeId === 'v360:jade_lamp') {
                        block.dimension.spawnItem(new ItemStack('v360:jade_shard', 1), block.center())
                    }
                    block.setType('v360:empty_crystal_lamp')
                    block.dimension.playSound('break.amethyst_cluster', block.location)
                }
            }
            else if (block.hasTag('v360:crystal_lamp0')) {
                block.setType('v360:empty_crystal_lamp')
                block.dimension.playSound('break.amethyst_cluster', block.location)
            }
        }
    })
})

world.beforeEvents.worldInitialize.subscribe(initEvent => {
    initEvent.blockComponentRegistry.registerCustomComponent('v360:rainbow_crystal_lamp', {
        onTick({ block }) {
            block.setType('v360:empty_rainbow_lamp')
            block.dimension.playSound('block.copper_bulb.turn_off', block.location, { volume: 3, pitch: 1 })
        },
        onPlace({ block }) {
            const { x, y, z } = block.location
            block.dimension.spawnEntity('v360:crystal_golem_activator', block.center()).triggerEvent('amethyst_activator')
            block.dimension.spawnEntity('v360:crystal_golem_activator', block.center()).triggerEvent('citrine_activator')
            block.dimension.spawnEntity('v360:crystal_golem_activator', block.center()).triggerEvent('garnet_activator')
            block.dimension.spawnEntity('v360:crystal_golem_activator', block.center()).triggerEvent('jade_activator')
            block.dimension.spawnParticle('v360:rainbow_lamp', { x: x, y: y + 0.5, z: z })
            block.dimension.playSound('resonate.amethyst_block', block.location)
        }
    })
})

world.afterEvents.entityHitEntity.subscribe(data => {
    const damager = data.damagingEntity
    const target = data.hitEntity
    const { x, y, z } = damager?.getViewDirection()
    if (damager?.typeId.includes('crystal_golem') && target?.typeId === 'minecraft:player')
        target.applyKnockback(+ x, + z, 3, 0.5)
})