import { world, system, Player, BlockPermutation, ItemStack } from '@minecraft/server'

world.beforeEvents.worldInitialize.subscribe(eventData => {
    eventData.blockComponentRegistry.registerCustomComponent('v360:bronze_door_interact', {
        onPlayerInteract(e) {
            const { block, player } = e
            const isUpperBlock = block.permutation.getState('v360:upper_block_bit')
            const equipment = player.getComponent('equippable')
            const selectedItem = equipment.getEquipment('Mainhand')
            const targetBlock = isUpperBlock ? block.below() : block.above()
            const currentOpenState = block.permutation.getState('v360:open_bit')
            const targetOpenState = targetBlock.permutation.getState('v360:open_bit')
            const newOpenState = !currentOpenState
            const newTargetOpenState = !targetOpenState
            const newBlockPermutation = BlockPermutation.resolve(block.typeId, {
                ...block.permutation.getAllStates(),
                'v360:open_bit': newOpenState
            })

            const newTargetBlockPermutation = BlockPermutation.resolve(targetBlock.typeId, {
                ...targetBlock.permutation.getAllStates(),
                'v360:open_bit': newTargetOpenState
            })
            block.setPermutation(newBlockPermutation)
            targetBlock.setPermutation(newTargetBlockPermutation)
            const sound = currentOpenState ? 'open_door.copper' : 'close_door.copper'
            player.playSound(sound)
            if (selectedItem?.typeId === 'minecraft:water_bucket') {
                player.playSound('bucket.empty_water')
                if (player.getGameMode() !== "creative") {
                    equipment.setEquipment('Mainhand', new ItemStack('minecraft:bucket', 1))
                }
            }
            if (block.typeId === 'v360:bronze_door' && selectedItem?.typeId === 'minecraft:water_bucket') {
                const currentStates = block.permutation.getAllStates()
                const structureName = 'mystructure:bronze_door'
                const { x, y, z } = block.location
                world.structureManager.place(structureName, e.dimension, { x, y, z })
                const newBlock = e.dimension.getBlock({ x, y, z })
                const newStates = { ...newBlock.permutation.getAllStates(), ...currentStates }
                const newPermutation = BlockPermutation.resolve(newBlock.typeId, newStates)
                newBlock.setPermutation(newPermutation)
            }
        }
    })
})

const processedBlocks = new Map()

world.beforeEvents.worldInitialize.subscribe(eventData => {
    eventData.blockComponentRegistry.registerCustomComponent('v360:bronze_door_place', {
        onPlace(e) {
            const { block } = e
            const aboveBlock = block.above()
            const northBlock = block.north()
            const southBlock = block.south()
            const eastBlock = block.east()
            const westBlock = block.west()
            const { x, y, z } = block.location
            if (processedBlocks.has(`${x},${y},${z}`)) {
                return
            }
            const currentStates = block.permutation.getAllStates()
            const cardinalDirection = currentStates['minecraft:cardinal_direction']
            if (aboveBlock?.typeId === 'minecraft:air' && !block?.permutation.getState('v360:upper_block_bit')) {
                const aboveBlockPermutation = BlockPermutation.resolve('v360:bronze_door', {
                    'v360:upper_block_bit': true,
                    'minecraft:cardinal_direction': cardinalDirection
                })
                aboveBlock.setPermutation(aboveBlockPermutation)
                const newPermutation = BlockPermutation.resolve(block.typeId, currentStates)
                block.setPermutation(newPermutation)
            }
            if (cardinalDirection === 'south' && eastBlock?.typeId === 'v360:bronze_door') {
                const eastBlockStates = eastBlock.permutation.getAllStates()
                if (!eastBlockStates['v360:door_hinge_bit']) {
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'v360:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const eastBlockAbove = aboveBlock.east();
                if (
                    aboveBlockCardinalDirection === 'south' &&
                    eastBlockAbove?.typeId === 'v360:bronze_door'
                ) {
                    const eastBlockAboveStates = eastBlockAbove.permutation.getAllStates()
                    if (!eastBlockAboveStates['v360:door_hinge_bit']) {
                        // Change the v360:door_hinge_bit state of the block above to true
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...eastBlockAboveStates,
                            'v360:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }

            // Check if the cardinal direction of the block is east
            if (cardinalDirection === 'east' && northBlock?.typeId === 'v360:bronze_door') {
                const northBlockStates = northBlock.permutation.getAllStates();
                // Check if the north block is a v360:door with v360:door_hinge_bit set to false
                if (!northBlockStates['v360:door_hinge_bit']) {
                    // Change the v360:door_hinge_bit state of the placed block to true
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'v360:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                // Also, check if the block above has the same cardinal direction and if the block to the north of the above block is a v360:door
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const northBlockAbove = aboveBlock.north();
                if (
                    aboveBlockCardinalDirection === 'east' &&
                    northBlockAbove?.typeId === 'v360:bronze_door'
                ) {
                    const northBlockAboveStates = northBlockAbove.permutation.getAllStates();
                    // Check if the north block above is a v360:door with v360:door_hinge_bit set to false
                    if (!northBlockAboveStates['v360:door_hinge_bit']) {
                        // Change the v360:door_hinge_bit state of the block above to true
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...northBlockAboveStates,
                            'v360:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }

            // Check if the cardinal direction of the block is north
            if (cardinalDirection === 'north' && westBlock?.typeId === 'v360:bronze_door') {
                const westBlockStates = westBlock.permutation.getAllStates();
                // Check if the west block is a v360:door with v360:door_hinge_bit set to false
                if (!westBlockStates['v360:door_hinge_bit']) {
                    // Change the v360:door_hinge_bit state of the placed block to true
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'v360:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                // Also, check if the block above has the same cardinal direction and if the block to the west of the above block is a v360:door
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const westBlockAbove = aboveBlock.west();
                if (
                    aboveBlockCardinalDirection === 'north' &&
                    westBlockAbove?.typeId === 'v360:bronze_door'
                ) {
                    const westBlockAboveStates = westBlockAbove.permutation.getAllStates();
                    // Check if the west block above is a v360:door with v360:door_hinge_bit set to false
                    if (!westBlockAboveStates['v360:door_hinge_bit']) {
                        // Change the v360:door_hinge_bit state of the block above to true
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...westBlockAboveStates,
                            'v360:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }

            // Check if the cardinal direction of the block is west
            if (cardinalDirection === 'west' && southBlock?.typeId === 'v360:bronze_door') {
                const southBlockStates = southBlock.permutation.getAllStates();
                // Check if the south block is a v360:door with v360:door_hinge_bit set to false
                if (!southBlockStates['v360:door_hinge_bit']) {
                    // Change the v360:door_hinge_bit state of the placed block to true
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'v360:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                // Also, check if the block above has the same cardinal direction and if the block to the south of the above block is a v360:door
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const southBlockAbove = aboveBlock.south();
                if (
                    aboveBlockCardinalDirection === 'west' &&
                    southBlockAbove?.typeId === 'v360:bronze_door'
                ) {
                    const southBlockAboveStates = southBlockAbove.permutation.getAllStates();
                    // Check if the south block above is a v360:door with v360:door_hinge_bit set to false
                    if (!southBlockAboveStates['v360:door_hinge_bit']) {
                        // Change the v360:door_hinge_bit state of the block above to true
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...southBlockAboveStates,
                            'v360:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }

            // Mark the block as processed
            processedBlocks.set(`${x},${y},${z}`, true);
        }
    });
});

// Subscribe to the 'playerBreakBlock' event to remove processed block entry
world.afterEvents.playerBreakBlock.subscribe(eventData => {
    const { block } = eventData;
    const { x, y, z } = block.location;

    // Remove the block's entry from the processedBlocks map
    processedBlocks.delete(`${x},${y},${z}`);

    // Check if the block above or below the destroyed block is also v360:door
    const aboveBlock = block.above();
    const belowBlock = block.below();
    if (belowBlock.typeId === 'v360:bronze_door') {
        belowBlock.setType('minecraft:air');
        processedBlocks.delete(`${x},${y - 1},${z}`); // Remove the below block's entry
    } else if (aboveBlock.typeId === 'v360:bronze_door') {
        aboveBlock.setType('minecraft:air');
        processedBlocks.delete(`${x},${y + 1},${z}`); // Remove the above block's entry
    }
});

world.beforeEvents.itemUseOn.subscribe(event => {
    const { block, itemStack } = event;
    const aboveBlock = block.above(2);
    const placedBlock = itemStack?.typeId === 'v360:bronze_door_item'

    if (aboveBlock.typeId !== 'minecraft:air' && placedBlock) {
        event.cancel = true;
    }
});

const blockStates = new Map();

// Subscribe to the 'worldInitialize' event to register custom components
world.beforeEvents.worldInitialize.subscribe(eventData => {
    // Register a custom component named v360:on_tick for trapdoor interaction with redstone
    eventData.blockComponentRegistry.registerCustomComponent('v360:bronze_door_tick', {
        onTick(e) {
            // Destructure event data for easier access
            const { block } = e;
            const { x, y, z } = block.location;
            const blockKey = `${x},${y},${z}`;
            const currentState = block.permutation.getState('v360:open_bit');
            const sound = currentState ? 'open_door.copper' : 'close_door.copper';

            // Get adjacent blocks
            const adjacentBlocks = {
                north: block.north(),
                east: block.east(),
                south: block.south(),
                west: block.west(),
                above: block.above(),
                below: block.below()
            };

            // Define blocks excluded from redstone functionality
            const excludedBlocks = [
                block.typeId.includes('door'), block.typeId.includes('trapdoor'), 'minecraft:observer', 'minecraft:unpowered_repeater',
                'minecraft:powered_repeater', 'minecraft:unpowered_comparator', 'minecraft:powered_comparator'
            ];

            // Check if any adjacent blocks have redstone power, excluding certain block types
            const hasRedstonePower = Object.values(adjacentBlocks).some(adjacentBlock =>
                !excludedBlocks.includes(adjacentBlock?.typeId) && (adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 1 }) || adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 2 }) || adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 3 }) || adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 4 }) || adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 5 }) || adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 6 }) || adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 7 }) || adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 8 }) || adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 9 }) || adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 10 }) || adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 11 }) || adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 12 }) || adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 13 }) || adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 14 }) || adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 15 }) || adjacentBlock?.typeId === 'minecraft:redstone_block' || adjacentBlock?.typeId === 'minecraft:redstone_torch') || adjacentBlock?.permutation.matches('lever', { 'open_bit': true }) || adjacentBlock?.permutation.matches('tripwire_hook', { 'powered_bit': true }) || adjacentBlock?.permutation.matches('detector_rail', { 'rail_data_bit': true })
            )

            // Check if there's a redstone torch directly above the block
            const isRedstoneTorchTop = adjacentBlocks.above?.typeId === 'minecraft:redstone_torch' &&
                adjacentBlocks.above.permutation.getState('torch_facing_direction') === 'top';

            // Define blocks that should be destroyed when receiving redstone power and in a specific position
            const destroyableBlocksArray = [
                block.typeId.includes('button'), block.typeId.includes('pressure_plate')
            ];

            // Check if the block above is a destroyable block and is powered
            const destroyableBlocks = destroyableBlocksArray.includes(adjacentBlocks.above?.typeId)

            // Check if an adjacent observer is facing the block and is powered
            const observerFacingBlock = [
                { block: adjacentBlocks.north, direction: 'north' },
                { block: adjacentBlocks.east, direction: 'east' },
                { block: adjacentBlocks.south, direction: 'south' },
                { block: adjacentBlocks.west, direction: 'west' },
                { block: adjacentBlocks.above, direction: 'up' },
                { block: adjacentBlocks.below, direction: 'down' }
            ].some(({ block, direction }) => block?.typeId === 'minecraft:observer' &&
                block?.permutation.getState('minecraft:facing_direction') === direction && block?.permutation.matches('observer', { 'powered_bit': true }));

            // Check if an adjacent powered repeater is facing the block
            const poweredRepeater = [
                { block: adjacentBlocks.north, direction: 'north' },
                { block: adjacentBlocks.east, direction: 'east' },
                { block: adjacentBlocks.south, direction: 'south' },
                { block: adjacentBlocks.west, direction: 'west' }
            ].some(({ block, direction }) => block?.typeId === 'minecraft:powered_repeater' &&
                block?.permutation.getState('minecraft:cardinal_direction') === direction);

            // Check if an adjacent powered comparator is facing the block
            const poweredComparator = [
                { block: adjacentBlocks.north, direction: 'north' },
                { block: adjacentBlocks.east, direction: 'east' },
                { block: adjacentBlocks.south, direction: 'south' },
                { block: adjacentBlocks.west, direction: 'west' }
            ].some(({ block, direction }) => block?.typeId === 'minecraft:powered_comparator' &&
                block?.permutation.getState('minecraft:cardinal_direction') === direction);

            // Retrieve the previous state of the block from the Map
            const previousState = blockStates.get(blockKey) || false;

            // Determine if the block should open or close based on redstone power and other conditions
            const shouldOpen = (hasRedstonePower || observerFacingBlock || poweredRepeater || poweredComparator) && !previousState && !isRedstoneTorchTop;
            const shouldClose = (!hasRedstonePower && !observerFacingBlock && !poweredRepeater && !poweredComparator) && previousState && !isRedstoneTorchTop;

            // Update the block's state if necessary
            if (shouldOpen || shouldClose) {
                const newState = shouldOpen;
                block.setPermutation(block.permutation.withState('v360:open_bit', newState));
                block.dimension.playSound(sound, block.location);
                blockStates.set(blockKey, newState);

                // Additional logic to update the corresponding part of the door
                const isUpperBlock = block.permutation.getState('v360:upper_block_bit');
                const correspondingBlock = isUpperBlock ? adjacentBlocks.below : adjacentBlocks.above;

                if (correspondingBlock?.typeId === 'v360:bronze_door') {
                    correspondingBlock.setPermutation(correspondingBlock.permutation.withState('v360:open_bit', newState));
                }
            }
            if (destroyableBlocks) {
                block.dimension.runCommand(`/setblock ${x} ${y + 1} ${z} air destroy`);
            }
        }
    });
})










world.beforeEvents.worldInitialize.subscribe(eventData => {
    eventData.blockComponentRegistry.registerCustomComponent('v360:araucaria_door_interact', {
        onPlayerInteract(e) {
            const { block, player } = e
            const isUpperBlock = block.permutation.getState('v360:upper_block_bit')
            const equipment = player.getComponent('equippable')
            const selectedItem = equipment.getEquipment('Mainhand')
            const targetBlock = isUpperBlock ? block.below() : block.above()
            const currentOpenState = block.permutation.getState('v360:open_bit')
            const targetOpenState = targetBlock.permutation.getState('v360:open_bit')
            const newOpenState = !currentOpenState
            const newTargetOpenState = !targetOpenState
            const newBlockPermutation = BlockPermutation.resolve(block.typeId, {
                ...block.permutation.getAllStates(),
                'v360:open_bit': newOpenState
            })

            const newTargetBlockPermutation = BlockPermutation.resolve(targetBlock.typeId, {
                ...targetBlock.permutation.getAllStates(),
                'v360:open_bit': newTargetOpenState
            })
            block.setPermutation(newBlockPermutation)
            targetBlock.setPermutation(newTargetBlockPermutation)
            const sound = currentOpenState ? 'open.wooden_door' : 'close.wooden_door'
            player.playSound(sound)
            if (selectedItem?.typeId === 'minecraft:water_bucket') {
                player.playSound('bucket.empty_water')
                if (player.getGameMode() !== "creative") {
                    equipment.setEquipment('Mainhand', new ItemStack('minecraft:bucket', 1))
                }
            }
            if (block.typeId === 'v360:araucaria_door' && selectedItem?.typeId === 'minecraft:water_bucket') {
                const currentStates = block.permutation.getAllStates()
                const structureName = 'mystructure:araucaria_door'
                const { x, y, z } = block.location
                world.structureManager.place(structureName, e.dimension, { x, y, z })
                const newBlock = e.dimension.getBlock({ x, y, z })
                const newStates = { ...newBlock.permutation.getAllStates(), ...currentStates }
                const newPermutation = BlockPermutation.resolve(newBlock.typeId, newStates)
                newBlock.setPermutation(newPermutation)
            }
        }
    })
})

world.beforeEvents.worldInitialize.subscribe(eventData => {
    eventData.blockComponentRegistry.registerCustomComponent('v360:araucaria_door_place', {
        onPlace(e) {
            const { block } = e
            const aboveBlock = block.above()
            const northBlock = block.north()
            const southBlock = block.south()
            const eastBlock = block.east()
            const westBlock = block.west()
            const { x, y, z } = block.location
            if (processedBlocks.has(`${x},${y},${z}`)) {
                return
            }
            const currentStates = block.permutation.getAllStates()
            const cardinalDirection = currentStates['minecraft:cardinal_direction']
            if (aboveBlock?.typeId === 'minecraft:air' && !block?.permutation.getState('v360:upper_block_bit')) {
                const aboveBlockPermutation = BlockPermutation.resolve('v360:araucaria_door', {
                    'v360:upper_block_bit': true,
                    'minecraft:cardinal_direction': cardinalDirection
                })
                aboveBlock.setPermutation(aboveBlockPermutation)
                const newPermutation = BlockPermutation.resolve(block.typeId, currentStates)
                block.setPermutation(newPermutation)
            }
            if (cardinalDirection === 'south' && eastBlock?.typeId === 'v360:araucaria_door') {
                const eastBlockStates = eastBlock.permutation.getAllStates()
                if (!eastBlockStates['v360:door_hinge_bit']) {
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'v360:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const eastBlockAbove = aboveBlock.east();
                if (
                    aboveBlockCardinalDirection === 'south' &&
                    eastBlockAbove?.typeId === 'v360:araucaria_door'
                ) {
                    const eastBlockAboveStates = eastBlockAbove.permutation.getAllStates()
                    if (!eastBlockAboveStates['v360:door_hinge_bit']) {
                        // Change the v360:door_hinge_bit state of the block above to true
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...eastBlockAboveStates,
                            'v360:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }

            // Check if the cardinal direction of the block is east
            if (cardinalDirection === 'east' && northBlock?.typeId === 'v360:araucaria_door') {
                const northBlockStates = northBlock.permutation.getAllStates();
                // Check if the north block is a v360:door with v360:door_hinge_bit set to false
                if (!northBlockStates['v360:door_hinge_bit']) {
                    // Change the v360:door_hinge_bit state of the placed block to true
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'v360:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                // Also, check if the block above has the same cardinal direction and if the block to the north of the above block is a v360:door
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const northBlockAbove = aboveBlock.north();
                if (
                    aboveBlockCardinalDirection === 'east' &&
                    northBlockAbove?.typeId === 'v360:araucaria_door'
                ) {
                    const northBlockAboveStates = northBlockAbove.permutation.getAllStates();
                    // Check if the north block above is a v360:door with v360:door_hinge_bit set to false
                    if (!northBlockAboveStates['v360:door_hinge_bit']) {
                        // Change the v360:door_hinge_bit state of the block above to true
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...northBlockAboveStates,
                            'v360:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }

            // Check if the cardinal direction of the block is north
            if (cardinalDirection === 'north' && westBlock?.typeId === 'v360:araucaria_door') {
                const westBlockStates = westBlock.permutation.getAllStates();
                // Check if the west block is a v360:door with v360:door_hinge_bit set to false
                if (!westBlockStates['v360:door_hinge_bit']) {
                    // Change the v360:door_hinge_bit state of the placed block to true
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'v360:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                // Also, check if the block above has the same cardinal direction and if the block to the west of the above block is a v360:door
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const westBlockAbove = aboveBlock.west();
                if (
                    aboveBlockCardinalDirection === 'north' &&
                    westBlockAbove?.typeId === 'v360:araucaria_door'
                ) {
                    const westBlockAboveStates = westBlockAbove.permutation.getAllStates();
                    // Check if the west block above is a v360:door with v360:door_hinge_bit set to false
                    if (!westBlockAboveStates['v360:door_hinge_bit']) {
                        // Change the v360:door_hinge_bit state of the block above to true
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...westBlockAboveStates,
                            'v360:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }

            // Check if the cardinal direction of the block is west
            if (cardinalDirection === 'west' && southBlock?.typeId === 'v360:araucaria_door') {
                const southBlockStates = southBlock.permutation.getAllStates();
                // Check if the south block is a v360:door with v360:door_hinge_bit set to false
                if (!southBlockStates['v360:door_hinge_bit']) {
                    // Change the v360:door_hinge_bit state of the placed block to true
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'v360:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                // Also, check if the block above has the same cardinal direction and if the block to the south of the above block is a v360:door
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const southBlockAbove = aboveBlock.south();
                if (
                    aboveBlockCardinalDirection === 'west' &&
                    southBlockAbove?.typeId === 'v360:araucaria_door'
                ) {
                    const southBlockAboveStates = southBlockAbove.permutation.getAllStates();
                    // Check if the south block above is a v360:door with v360:door_hinge_bit set to false
                    if (!southBlockAboveStates['v360:door_hinge_bit']) {
                        // Change the v360:door_hinge_bit state of the block above to true
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...southBlockAboveStates,
                            'v360:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }

            // Mark the block as processed
            processedBlocks.set(`${x},${y},${z}`, true);
        }
    });
});

// Subscribe to the 'playerBreakBlock' event to remove processed block entry
world.afterEvents.playerBreakBlock.subscribe(eventData => {
    const { block } = eventData;
    const { x, y, z } = block.location;

    // Remove the block's entry from the processedBlocks map
    processedBlocks.delete(`${x},${y},${z}`);

    // Check if the block above or below the destroyed block is also v360:door
    const aboveBlock = block.above();
    const belowBlock = block.below();
    if (belowBlock.typeId === 'v360:araucaria_door') {
        belowBlock.setType('minecraft:air');
        processedBlocks.delete(`${x},${y - 1},${z}`); // Remove the below block's entry
    } else if (aboveBlock.typeId === 'v360:araucaria_door') {
        aboveBlock.setType('minecraft:air');
        processedBlocks.delete(`${x},${y + 1},${z}`); // Remove the above block's entry
    }
});

world.beforeEvents.itemUseOn.subscribe(event => {
    const { block, itemStack } = event;
    const aboveBlock = block.above(2);
    const placedBlock = itemStack?.typeId === 'v360:araucaria_door_item'

    if (aboveBlock.typeId !== 'minecraft:air' && placedBlock) {
        event.cancel = true;
    }
});

// Subscribe to the 'worldInitialize' event to register custom components
world.beforeEvents.worldInitialize.subscribe(eventData => {
    // Register a custom component named v360:on_tick for trapdoor interaction with redstone
    eventData.blockComponentRegistry.registerCustomComponent('v360:araucaria_door_tick', {
        onTick(e) {
            // Destructure event data for easier access
            const { block } = e;
            const { x, y, z } = block.location;
            const blockKey = `${x},${y},${z}`;
            const currentState = block.permutation.getState('v360:open_bit');
            const sound = currentState ? 'open.wooden_door' : 'close.wooden_door';

            // Get adjacent blocks
            const adjacentBlocks = {
                north: block.north(),
                east: block.east(),
                south: block.south(),
                west: block.west(),
                above: block.above(),
                below: block.below()
            };

            // Define blocks excluded from redstone functionality
            const excludedBlocks = [
                block.typeId.includes('door'), block.typeId.includes('trapdoor'), 'minecraft:observer', 'minecraft:unpowered_repeater',
                'minecraft:powered_repeater', 'minecraft:unpowered_comparator', 'minecraft:powered_comparator'
            ];

            // Check if any adjacent blocks have redstone power, excluding certain block types
            const hasRedstonePower = Object.values(adjacentBlocks).some(adjacentBlock =>
                !excludedBlocks.includes(adjacentBlock?.typeId) && (adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 1 }) || adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 2 }) || adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 3 }) || adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 4 }) || adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 5 }) || adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 6 }) || adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 7 }) || adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 8 }) || adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 9 }) || adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 10 }) || adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 11 }) || adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 12 }) || adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 13 }) || adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 14 }) || adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 15 }) || adjacentBlock?.typeId === 'minecraft:redstone_block' || adjacentBlock?.typeId === 'minecraft:redstone_torch') || adjacentBlock?.permutation.matches('lever', { 'open_bit': true }) || adjacentBlock?.permutation.matches('tripwire_hook', { 'powered_bit': true }) || adjacentBlock?.permutation.matches('detector_rail', { 'rail_data_bit': true })
            )

            // Check if there's a redstone torch directly above the block
            const isRedstoneTorchTop = adjacentBlocks.above?.typeId === 'minecraft:redstone_torch' &&
                adjacentBlocks.above.permutation.getState('torch_facing_direction') === 'top';

            // Define blocks that should be destroyed when receiving redstone power and in a specific position
            const destroyableBlocksArray = [
                block.typeId.includes('button'), block.typeId.includes('pressure_plate')
            ];

            // Check if the block above is a destroyable block and is powered
            const destroyableBlocks = destroyableBlocksArray.includes(adjacentBlocks.above?.typeId)

            // Check if an adjacent observer is facing the block and is powered
            const observerFacingBlock = [
                { block: adjacentBlocks.north, direction: 'north' },
                { block: adjacentBlocks.east, direction: 'east' },
                { block: adjacentBlocks.south, direction: 'south' },
                { block: adjacentBlocks.west, direction: 'west' },
                { block: adjacentBlocks.above, direction: 'up' },
                { block: adjacentBlocks.below, direction: 'down' }
            ].some(({ block, direction }) => block?.typeId === 'minecraft:observer' &&
                block?.permutation.getState('minecraft:facing_direction') === direction && block?.permutation.matches('observer', { 'powered_bit': true }));

            // Check if an adjacent powered repeater is facing the block
            const poweredRepeater = [
                { block: adjacentBlocks.north, direction: 'north' },
                { block: adjacentBlocks.east, direction: 'east' },
                { block: adjacentBlocks.south, direction: 'south' },
                { block: adjacentBlocks.west, direction: 'west' }
            ].some(({ block, direction }) => block?.typeId === 'minecraft:powered_repeater' &&
                block?.permutation.getState('minecraft:cardinal_direction') === direction);

            // Check if an adjacent powered comparator is facing the block
            const poweredComparator = [
                { block: adjacentBlocks.north, direction: 'north' },
                { block: adjacentBlocks.east, direction: 'east' },
                { block: adjacentBlocks.south, direction: 'south' },
                { block: adjacentBlocks.west, direction: 'west' }
            ].some(({ block, direction }) => block?.typeId === 'minecraft:powered_comparator' &&
                block?.permutation.getState('minecraft:cardinal_direction') === direction);

            // Retrieve the previous state of the block from the Map
            const previousState = blockStates.get(blockKey) || false;

            // Determine if the block should open or close based on redstone power and other conditions
            const shouldOpen = (hasRedstonePower || observerFacingBlock || poweredRepeater || poweredComparator) && !previousState && !isRedstoneTorchTop;
            const shouldClose = (!hasRedstonePower && !observerFacingBlock && !poweredRepeater && !poweredComparator) && previousState && !isRedstoneTorchTop;

            // Update the block's state if necessary
            if (shouldOpen || shouldClose) {
                const newState = shouldOpen;
                block.setPermutation(block.permutation.withState('v360:open_bit', newState));
                block.dimension.playSound(sound, block.location);
                blockStates.set(blockKey, newState);

                // Additional logic to update the corresponding part of the door
                const isUpperBlock = block.permutation.getState('v360:upper_block_bit');
                const correspondingBlock = isUpperBlock ? adjacentBlocks.below : adjacentBlocks.above;

                if (correspondingBlock?.typeId === 'v360:araucaria_door') {
                    correspondingBlock.setPermutation(correspondingBlock.permutation.withState('v360:open_bit', newState));
                }
            }
            if (destroyableBlocks) {
                block.dimension.runCommand(`/setblock ${x} ${y + 1} ${z} air destroy`);
            }
        }
    });
})










world.beforeEvents.worldInitialize.subscribe(eventData => {
    eventData.blockComponentRegistry.registerCustomComponent('v360:end_door_interact', {
        onPlayerInteract(e) {
            const { block, player } = e
            const isUpperBlock = block.permutation.getState('v360:upper_block_bit')
            const equipment = player.getComponent('equippable')
            const selectedItem = equipment.getEquipment('Mainhand')
            const targetBlock = isUpperBlock ? block.below() : block.above()
            const currentOpenState = block.permutation.getState('v360:open_bit')
            const targetOpenState = targetBlock.permutation.getState('v360:open_bit')
            const newOpenState = !currentOpenState
            const newTargetOpenState = !targetOpenState
            const newBlockPermutation = BlockPermutation.resolve(block.typeId, {
                ...block.permutation.getAllStates(),
                'v360:open_bit': newOpenState
            })

            const newTargetBlockPermutation = BlockPermutation.resolve(targetBlock.typeId, {
                ...targetBlock.permutation.getAllStates(),
                'v360:open_bit': newTargetOpenState
            })
            block.setPermutation(newBlockPermutation)
            targetBlock.setPermutation(newTargetBlockPermutation)
            const sound = currentOpenState ? 'open.wooden_door' : 'close.wooden_door'
            player.playSound(sound)
            if (selectedItem?.typeId === 'minecraft:water_bucket') {
                player.playSound('bucket.empty_water')
                if (player.getGameMode() !== "creative") {
                    equipment.setEquipment('Mainhand', new ItemStack('minecraft:bucket', 1))
                }
            }
            if (block.typeId === 'v360:end_door' && selectedItem?.typeId === 'minecraft:water_bucket') {
                const currentStates = block.permutation.getAllStates()
                const structureName = 'mystructure:end_door'
                const { x, y, z } = block.location
                world.structureManager.place(structureName, e.dimension, { x, y, z })
                const newBlock = e.dimension.getBlock({ x, y, z })
                const newStates = { ...newBlock.permutation.getAllStates(), ...currentStates }
                const newPermutation = BlockPermutation.resolve(newBlock.typeId, newStates)
                newBlock.setPermutation(newPermutation)
            }
        }
    })
})

world.beforeEvents.worldInitialize.subscribe(eventData => {
    eventData.blockComponentRegistry.registerCustomComponent('v360:end_door_place', {
        onPlace(e) {
            const { block } = e
            const aboveBlock = block.above()
            const northBlock = block.north()
            const southBlock = block.south()
            const eastBlock = block.east()
            const westBlock = block.west()
            const { x, y, z } = block.location
            if (processedBlocks.has(`${x},${y},${z}`)) {
                return
            }
            const currentStates = block.permutation.getAllStates()
            const cardinalDirection = currentStates['minecraft:cardinal_direction']
            if (aboveBlock?.typeId === 'minecraft:air' && !block?.permutation.getState('v360:upper_block_bit')) {
                const aboveBlockPermutation = BlockPermutation.resolve('v360:end_door', {
                    'v360:upper_block_bit': true,
                    'minecraft:cardinal_direction': cardinalDirection
                })
                aboveBlock.setPermutation(aboveBlockPermutation)
                const newPermutation = BlockPermutation.resolve(block.typeId, currentStates)
                block.setPermutation(newPermutation)
            }
            if (cardinalDirection === 'south' && eastBlock?.typeId === 'v360:end_door') {
                const eastBlockStates = eastBlock.permutation.getAllStates()
                if (!eastBlockStates['v360:door_hinge_bit']) {
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'v360:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const eastBlockAbove = aboveBlock.east();
                if (
                    aboveBlockCardinalDirection === 'south' &&
                    eastBlockAbove?.typeId === 'v360:end_door'
                ) {
                    const eastBlockAboveStates = eastBlockAbove.permutation.getAllStates()
                    if (!eastBlockAboveStates['v360:door_hinge_bit']) {
                        // Change the v360:door_hinge_bit state of the block above to true
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...eastBlockAboveStates,
                            'v360:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }

            // Check if the cardinal direction of the block is east
            if (cardinalDirection === 'east' && northBlock?.typeId === 'v360:end_door') {
                const northBlockStates = northBlock.permutation.getAllStates();
                // Check if the north block is a v360:door with v360:door_hinge_bit set to false
                if (!northBlockStates['v360:door_hinge_bit']) {
                    // Change the v360:door_hinge_bit state of the placed block to true
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'v360:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                // Also, check if the block above has the same cardinal direction and if the block to the north of the above block is a v360:door
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const northBlockAbove = aboveBlock.north();
                if (
                    aboveBlockCardinalDirection === 'east' &&
                    northBlockAbove?.typeId === 'v360:end_door'
                ) {
                    const northBlockAboveStates = northBlockAbove.permutation.getAllStates();
                    // Check if the north block above is a v360:door with v360:door_hinge_bit set to false
                    if (!northBlockAboveStates['v360:door_hinge_bit']) {
                        // Change the v360:door_hinge_bit state of the block above to true
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...northBlockAboveStates,
                            'v360:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }

            // Check if the cardinal direction of the block is north
            if (cardinalDirection === 'north' && westBlock?.typeId === 'v360:end_door') {
                const westBlockStates = westBlock.permutation.getAllStates();
                // Check if the west block is a v360:door with v360:door_hinge_bit set to false
                if (!westBlockStates['v360:door_hinge_bit']) {
                    // Change the v360:door_hinge_bit state of the placed block to true
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'v360:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                // Also, check if the block above has the same cardinal direction and if the block to the west of the above block is a v360:door
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const westBlockAbove = aboveBlock.west();
                if (
                    aboveBlockCardinalDirection === 'north' &&
                    westBlockAbove?.typeId === 'v360:end_door'
                ) {
                    const westBlockAboveStates = westBlockAbove.permutation.getAllStates();
                    // Check if the west block above is a v360:door with v360:door_hinge_bit set to false
                    if (!westBlockAboveStates['v360:door_hinge_bit']) {
                        // Change the v360:door_hinge_bit state of the block above to true
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...westBlockAboveStates,
                            'v360:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }

            // Check if the cardinal direction of the block is west
            if (cardinalDirection === 'west' && southBlock?.typeId === 'v360:end_door') {
                const southBlockStates = southBlock.permutation.getAllStates();
                // Check if the south block is a v360:door with v360:door_hinge_bit set to false
                if (!southBlockStates['v360:door_hinge_bit']) {
                    // Change the v360:door_hinge_bit state of the placed block to true
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'v360:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                // Also, check if the block above has the same cardinal direction and if the block to the south of the above block is a v360:door
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const southBlockAbove = aboveBlock.south();
                if (
                    aboveBlockCardinalDirection === 'west' &&
                    southBlockAbove?.typeId === 'v360:end_door'
                ) {
                    const southBlockAboveStates = southBlockAbove.permutation.getAllStates();
                    // Check if the south block above is a v360:door with v360:door_hinge_bit set to false
                    if (!southBlockAboveStates['v360:door_hinge_bit']) {
                        // Change the v360:door_hinge_bit state of the block above to true
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...southBlockAboveStates,
                            'v360:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }

            // Mark the block as processed
            processedBlocks.set(`${x},${y},${z}`, true);
        }
    });
});

// Subscribe to the 'playerBreakBlock' event to remove processed block entry
world.afterEvents.playerBreakBlock.subscribe(eventData => {
    const { block } = eventData;
    const { x, y, z } = block.location;

    // Remove the block's entry from the processedBlocks map
    processedBlocks.delete(`${x},${y},${z}`);

    // Check if the block above or below the destroyed block is also v360:door
    const aboveBlock = block.above();
    const belowBlock = block.below();
    if (belowBlock.typeId === 'v360:end_door') {
        belowBlock.setType('minecraft:air');
        processedBlocks.delete(`${x},${y - 1},${z}`); // Remove the below block's entry
    } else if (aboveBlock.typeId === 'v360:end_door') {
        aboveBlock.setType('minecraft:air');
        processedBlocks.delete(`${x},${y + 1},${z}`); // Remove the above block's entry
    }
});

world.beforeEvents.itemUseOn.subscribe(event => {
    const { block, itemStack } = event;
    const aboveBlock = block.above(2);
    const placedBlock = itemStack?.typeId === 'v360:end_door_item'

    if (aboveBlock.typeId !== 'minecraft:air' && placedBlock) {
        event.cancel = true;
    }
});

// Subscribe to the 'worldInitialize' event to register custom components
world.beforeEvents.worldInitialize.subscribe(eventData => {
    // Register a custom component named v360:on_tick for trapdoor interaction with redstone
    eventData.blockComponentRegistry.registerCustomComponent('v360:end_door_tick', {
        onTick(e) {
            // Destructure event data for easier access
            const { block } = e;
            const { x, y, z } = block.location;
            const blockKey = `${x},${y},${z}`;
            const currentState = block.permutation.getState('v360:open_bit');
            const sound = currentState ? 'open.wooden_door' : 'close.wooden_door';

            // Get adjacent blocks
            const adjacentBlocks = {
                north: block.north(),
                east: block.east(),
                south: block.south(),
                west: block.west(),
                above: block.above(),
                below: block.below()
            };

            // Define blocks excluded from redstone functionality
            const excludedBlocks = [
                block.typeId.includes('door'), block.typeId.includes('trapdoor'), 'minecraft:observer', 'minecraft:unpowered_repeater',
                'minecraft:powered_repeater', 'minecraft:unpowered_comparator', 'minecraft:powered_comparator'
            ];

            // Check if any adjacent blocks have redstone power, excluding certain block types
            const hasRedstonePower = Object.values(adjacentBlocks).some(adjacentBlock =>
                !excludedBlocks.includes(adjacentBlock?.typeId) && (adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 1 }) || adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 2 }) || adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 3 }) || adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 4 }) || adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 5 }) || adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 6 }) || adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 7 }) || adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 8 }) || adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 9 }) || adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 10 }) || adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 11 }) || adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 12 }) || adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 13 }) || adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 14 }) || adjacentBlock?.permutation.matches('minecraft:redstone_wire', { 'redstone_signal': 15 }) || adjacentBlock?.typeId === 'minecraft:redstone_block' || adjacentBlock?.typeId === 'minecraft:redstone_torch') || adjacentBlock?.permutation.matches('lever', { 'open_bit': true }) || adjacentBlock?.permutation.matches('tripwire_hook', { 'powered_bit': true }) || adjacentBlock?.permutation.matches('detector_rail', { 'rail_data_bit': true })
            )

            // Check if there's a redstone torch directly above the block
            const isRedstoneTorchTop = adjacentBlocks.above?.typeId === 'minecraft:redstone_torch' &&
                adjacentBlocks.above.permutation.getState('torch_facing_direction') === 'top';

            // Define blocks that should be destroyed when receiving redstone power and in a specific position
            const destroyableBlocksArray = [
                block.typeId.includes('button'), block.typeId.includes('pressure_plate')
            ];

            // Check if the block above is a destroyable block and is powered
            const destroyableBlocks = destroyableBlocksArray.includes(adjacentBlocks.above?.typeId)

            // Check if an adjacent observer is facing the block and is powered
            const observerFacingBlock = [
                { block: adjacentBlocks.north, direction: 'north' },
                { block: adjacentBlocks.east, direction: 'east' },
                { block: adjacentBlocks.south, direction: 'south' },
                { block: adjacentBlocks.west, direction: 'west' },
                { block: adjacentBlocks.above, direction: 'up' },
                { block: adjacentBlocks.below, direction: 'down' }
            ].some(({ block, direction }) => block?.typeId === 'minecraft:observer' &&
                block?.permutation.getState('minecraft:facing_direction') === direction && block?.permutation.matches('observer', { 'powered_bit': true }));

            // Check if an adjacent powered repeater is facing the block
            const poweredRepeater = [
                { block: adjacentBlocks.north, direction: 'north' },
                { block: adjacentBlocks.east, direction: 'east' },
                { block: adjacentBlocks.south, direction: 'south' },
                { block: adjacentBlocks.west, direction: 'west' }
            ].some(({ block, direction }) => block?.typeId === 'minecraft:powered_repeater' &&
                block?.permutation.getState('minecraft:cardinal_direction') === direction);

            // Check if an adjacent powered comparator is facing the block
            const poweredComparator = [
                { block: adjacentBlocks.north, direction: 'north' },
                { block: adjacentBlocks.east, direction: 'east' },
                { block: adjacentBlocks.south, direction: 'south' },
                { block: adjacentBlocks.west, direction: 'west' }
            ].some(({ block, direction }) => block?.typeId === 'minecraft:powered_comparator' &&
                block?.permutation.getState('minecraft:cardinal_direction') === direction);

            // Retrieve the previous state of the block from the Map
            const previousState = blockStates.get(blockKey) || false;

            // Determine if the block should open or close based on redstone power and other conditions
            const shouldOpen = (hasRedstonePower || observerFacingBlock || poweredRepeater || poweredComparator) && !previousState && !isRedstoneTorchTop;
            const shouldClose = (!hasRedstonePower && !observerFacingBlock && !poweredRepeater && !poweredComparator) && previousState && !isRedstoneTorchTop;

            // Update the block's state if necessary
            if (shouldOpen || shouldClose) {
                const newState = shouldOpen;
                block.setPermutation(block.permutation.withState('v360:open_bit', newState));
                block.dimension.playSound(sound, block.location);
                blockStates.set(blockKey, newState);

                // Additional logic to update the corresponding part of the door
                const isUpperBlock = block.permutation.getState('v360:upper_block_bit');
                const correspondingBlock = isUpperBlock ? adjacentBlocks.below : adjacentBlocks.above;

                if (correspondingBlock?.typeId === 'v360:end_door') {
                    correspondingBlock.setPermutation(correspondingBlock.permutation.withState('v360:open_bit', newState));
                }
            }
            if (destroyableBlocks) {
                block.dimension.runCommand(`/setblock ${x} ${y + 1} ${z} air destroy`);
            }
        }
    });
})