import { world, system, ItemStack, BlockPermutation, TicksPerSecond } from '@minecraft/server'

world.beforeEvents.worldInitialize.subscribe(initEvent => {
    initEvent.blockComponentRegistry.registerCustomComponent('v360:corolla', {
        onPlace({ block }) {
            if (block.permutation.matches('v360:corolla', { 'v360:double_plant': 'lower_bit' }) && !block.above().permutation.matches('v360:corolla', { 'v360:double_plant': 'upper_bit' }) && block?.above().isAir) {
                const aboveBlockPermutation = BlockPermutation.resolve('v360:corolla', {
                    'v360:double_plant': 'upper_bit',
                })
                block.above().setPermutation(aboveBlockPermutation)
            }
        },
        onPlayerDestroy({ block, destroyedBlockPermutation }) {
            const { x, y, z } = block.location
            if (block.above().permutation.matches('v360:corolla', { 'v360:double_plant': 'upper_bit' })) {
                block.above().setType('air')
            }
            if (block.below().permutation.matches('v360:corolla', { 'v360:double_plant': 'lower_bit' })) {
                block.below().setType('air')
            }
        },
        onTick({ block }) {
            if (block.permutation.matches('v360:corolla', { 'v360:double_plant': 'lower_bit' }) && !block.above().permutation.matches('v360:corolla', { 'v360:double_plant': 'upper_bit' }) && block?.above().isAir) {
                const aboveBlockPermutation = BlockPermutation.resolve('v360:corolla', {
                    'v360:double_plant': 'upper_bit',
                })
                block.above().setPermutation(aboveBlockPermutation)
            }
        }
    })
})

world.beforeEvents.itemUseOn.subscribe(event => {
    const { block, itemStack } = event
    const aboveBlock = block.above(2)
    const placedBlock = (itemStack?.typeId === 'v360:corolla_item' || itemStack?.typeId === 'v360:glowgloxinia_item')

    if (aboveBlock.typeId !== 'minecraft:air' && placedBlock) {
        event.cancel = true
    }
})

world.beforeEvents.worldInitialize.subscribe(initEvent => {
    initEvent.blockComponentRegistry.registerCustomComponent('v360:glowgloxinia', {
        onPlace({ block }) {
            if (block.permutation.matches('v360:glowgloxinia', { 'v360:double_plant': 'lower_bit' }) && !block.above().permutation.matches('v360:glowgloxinia', { 'v360:double_plant': 'upper_bit' }) && block?.above().isAir) {
                const aboveBlockPermutation = BlockPermutation.resolve('v360:glowgloxinia', {
                    'v360:double_plant': 'upper_bit',
                })
                block.above().setPermutation(aboveBlockPermutation)
            }
        },
        onPlayerDestroy({ block, destroyedBlockPermutation }) {
            const { x, y, z } = block.location
            if (block.above().permutation.matches('v360:glowgloxinia', { 'v360:double_plant': 'upper_bit' })) {
                block.above().setType('air')
            }
            if (block.below().permutation.matches('v360:glowgloxinia', { 'v360:double_plant': 'lower_bit' })) {
                block.below().setType('air')
            }
        },
        onTick({ block }) {
            block.dimension.getEntities({ location: block.location, maxDistance: 5 }).forEach(
                (data) => {
                    data.addEffect('speed', TicksPerSecond * 5, { showParticles: false, amplifier: 2 })
                }
            )
            if (block.permutation.matches('v360:glowgloxinia', { 'v360:double_plant': 'lower_bit' }) && !block.above().permutation.matches('v360:glowgloxinia', { 'v360:double_plant': 'upper_bit' }) && block?.above().isAir) {
                const aboveBlockPermutation = BlockPermutation.resolve('v360:glowgloxinia', {
                    'v360:double_plant': 'upper_bit',
                })
                block.above().setPermutation(aboveBlockPermutation)
            }
        }
    })
})

world.beforeEvents.playerBreakBlock.subscribe(data => {
    const block = data.block
    const { x, y, z } = block.location
    if (block.above(2).permutation.matches('v360:corolla', { 'v360:double_plant': 'upper_bit' }) || block.above(2).permutation.matches('v360:glowgloxinia', { 'v360:double_plant': 'upper_bit' })) {
        system.run(() => {
            block.dimension.runCommand(`fill ${x} ${y + 2} ${z} ${x} ${y + 2} ${z} air`)
        })
    }
})