import { world, ItemStack, BlockPermutation } from '@minecraft/server'

world.beforeEvents.worldInitialize.subscribe(eventData => {
    eventData.blockComponentRegistry.registerCustomComponent('v360:copper_lantern', {
        onPlayerInteract(e) {
            const { block, player } = e
            const equipment = player.getComponent('equippable')
            const selectedItem = equipment.getEquipment('Mainhand')
            if (selectedItem?.typeId === 'minecraft:water_bucket') {
                player.playSound('bucket.empty_water')
                if (player.getGameMode() !== "creative") {
                    equipment.setEquipment('Mainhand', new ItemStack('minecraft:bucket', 1));
                }
            }
            if (block.typeId === 'v360:copper_lantern' && selectedItem?.typeId === 'minecraft:water_bucket') {
                const blockFace = block.permutation.getState('minecraft:block_face')
                let structureName = '';
                if (blockFace === 'down') {
                    structureName = 'mystructure:hanging_copper_lantern'
                } else if (blockFace === 'up') {
                    structureName = 'mystructure:floor_copper_lantern'
                }
                if (structureName) {
                    const { x, y, z } = block
                    world.structureManager.place(structureName, e.dimension, { x, y, z })
                }
            }
        }
    })
})

world.beforeEvents.worldInitialize.subscribe(eventData => {
    eventData.blockComponentRegistry.registerCustomComponent('v360:unlit_lantern', {
        onPlayerInteract(e) {
            const { block, player } = e
            const equipment = player.getComponent('equippable')
            const selectedItem = equipment.getEquipment('Mainhand')
            const lanternUp = BlockPermutation.resolve('minecraft:lantern', { 'hanging': true })
            const lanternDown = BlockPermutation.resolve('minecraft:lantern', { 'hanging': false })
            const soulLanternUp = BlockPermutation.resolve('minecraft:soul_lantern', { 'hanging': true })
            const soulLanternDown = BlockPermutation.resolve('minecraft:soul_lantern', { 'hanging': false })
            const copperLanternUp = BlockPermutation.resolve('v360:copper_lantern', { 'minecraft:block_face': 'down' })
            const copperLanternDown = BlockPermutation.resolve('v360:copper_lantern', { 'minecraft:block_face': 'up' })
            const lushLanternUp = BlockPermutation.resolve('v360:lush_lantern', { 'minecraft:block_face': 'down' })
            const lushLanternDown = BlockPermutation.resolve('v360:lush_lantern', { 'minecraft:block_face': 'up' })
            if (selectedItem?.typeId === 'minecraft:water_bucket') {
                player.playSound('bucket.empty_water')
                if (player.getGameMode() !== "creative") {
                    equipment.setEquipment('Mainhand', new ItemStack('minecraft:bucket', 1));
                }
            }
            if (block.typeId === 'v360:unlit_lantern' && selectedItem?.typeId === 'minecraft:water_bucket') {
                const blockFace = block.permutation.getState('minecraft:block_face')
                let structureName = '';
                if (blockFace === 'down') {
                    structureName = 'mystructure:hanging_unlit_lantern'
                } else if (blockFace === 'up') {
                    structureName = 'mystructure:floor_unlit_lantern'
                }
                if (structureName) {
                    const { x, y, z } = block
                    world.structureManager.place(structureName, e.dimension, { x, y, z })
                }
            }
            else if (selectedItem?.typeId === 'minecraft:torch') {
                if (player.getGameMode() !== 'creative') {
                    player.runCommand(`clear @s torch 0 1`)
                }
                if (block.permutation.matches('v360:unlit_lantern', { 'minecraft:block_face': 'down' })) {
                    block.setPermutation(lanternUp)
                    block.dimension.playSound('dig.candle', block.location)
                }
                else if (block.permutation.matches('v360:unlit_lantern', { 'minecraft:block_face': 'up' })) {
                    block.setPermutation(lanternDown)
                    block.dimension.playSound('dig.candle', block.location)
                }
            }
            else if (selectedItem?.typeId === 'minecraft:soul_torch') {
                if (player.getGameMode() !== 'creative') {
                    player.runCommand(`clear @s soul_torch 0 1`)
                }
                if (block.permutation.matches('v360:unlit_lantern', { 'minecraft:block_face': 'down' })) {
                    block.setPermutation(soulLanternUp)
                    block.dimension.playSound('dig.candle', block.location)
                }
                else if (block.permutation.matches('v360:unlit_lantern', { 'minecraft:block_face': 'up' })) {
                    block.setPermutation(soulLanternDown)
                    block.dimension.playSound('dig.candle', block.location)
                }
            }
            else if (selectedItem?.typeId === 'v360:copper_torch') {
                if (player.getGameMode() !== 'creative') {
                    player.runCommand(`clear @s v360:copper_torch 0 1`)
                }
                if (block.permutation.matches('v360:unlit_lantern', { 'minecraft:block_face': 'down' })) {
                    block.setPermutation(copperLanternUp)
                    block.dimension.playSound('dig.candle', block.location)
                }
                else if (block.permutation.matches('v360:unlit_lantern', { 'minecraft:block_face': 'up' })) {
                    block.setPermutation(copperLanternDown)
                    block.dimension.playSound('dig.candle', block.location)
                }
            }
            else if (selectedItem?.typeId === 'minecraft:glow_berries') {
                if (player.getGameMode() !== 'creative') {
                    player.runCommand(`clear @s glow_berries 0 1`)
                }
                if (block.permutation.matches('v360:unlit_lantern', { 'minecraft:block_face': 'down' })) {
                    block.setPermutation(lushLanternUp)
                    block.dimension.playSound('dig.cave_vines', block.location)
                }
                else if (block.permutation.matches('v360:unlit_lantern', { 'minecraft:block_face': 'up' })) {
                    block.setPermutation(lushLanternDown)
                    block.dimension.playSound('dig.cave_vines', block.location)
                }
            }
        }
    })
})